from os.path import join
from pathlib import Path
from shutil import move
from zipfile import is_zipfile, ZipFile


def try_unzip(path):
    if isinstance(path, Path):
        path = str(path)
    if is_zipfile(path):
        from shutil import move
        name, ext = splitext(path)
        if ext == '.zip':
            zip_path = path
            path = join(path.parent, name)
        else:
            zip_path = join(path.parent, '%s.zip' % name)
            print('Renaming %s to %s' % (path, zip_path))
            move(path, zip_path)

        print('Extracting %s to %s' % (zip_path, path))
        with ZipFile(zip_path) as f:
            f.extractall(path)

        zip_path.unlink()

    return path



#!/usr/bin/env python

from argparse import ArgumentParser
from os import chdir
from os.path import exists


parser = ArgumentParser()
parser.add_argument('nbs',nargs='*',help='Optional: notebook to run, output path; default: "run.ipynb"')
parser.add_argument('--cwd',action='store_true',help="When set, run from the current dir (by default, the nb's parent dir is used)")
parser.add_argument('--progress',action='store_true',help="When set, have papermill show progress bars")
parser.add_argument('-d','--dir',help="Resolve paths relative to this directory")
parser.add_argument('-f','--file',help='YAML file with parameters to pass to notebook <nb>')
parser.add_argument('-y','--yaml',help='YAML string with parameters to pass to notebook <nb>')
parser.add_argument('-j','--json',help='JSON string with parameters to pass to notebook <nb>')
args = parser.parse_args()
nbs = args.nbs
out = None
if len(nbs) == 0:
    nb = 'run.ipynb'
elif len(nbs) == 1:
    [nb] = nbs
elif len(nbs) == 2:
    [nb,out] = nbs
else:
    raise ValueError(f'Too many positional args (max 2; input+output): {nbs}')

cwd = args.cwd
progress_bar = args.progress
dir = args.dir
if dir: chdir(dir)

file = args.file
yaml_str = args.yaml
json_str = args.json

params = {}

if not file and exists('run.yml'):
    file = 'run.yml'

if file:
    import yaml
    with open(file,'r') as f:
        params = { **yaml.safe_load(f), **params }

if yaml_str:
    import yaml
    params = { **yaml.safe_load(yaml_str), **params }

if json_str:
    import json
    params = { **json.loads(json_str), **params }


from .papermill import execute

execute(
    input=nb,
    output=out,
    cwd=cwd,
    progress_bar=progress_bar,
    **params
)



#!/usr/bin/env python
# coding: utf-8

# # imports
# Common Jupyter imports and helpers to wildcard import from other notebooks

# Helper for optional imports:

# In[7]:


from contextlib import suppress
_try = suppress(ImportError, ModuleNotFoundError)


# ## stdlib
# Common imports (and associated helpers) from the Python standard library:

# ### Date/Time

# In[ ]:


from dateutil.parser import parse
from datetime import datetime as dt, date
with _try: from pytz import UTC
now = dt.now()
ISO_DATE_FMT = '%Y-%m-%dT%H:%M:%S'
runtime = now.strftime(ISO_DATE_FMT)
today = now.strftime('%Y-%m-%d')


# ### Paths

# In[ ]:


from pathlib import Path
def mkdir(path, *args, exist_ok=True, **kwargs):
    os.makedirs(str(path), *args, exist_ok=exist_ok, **kwargs)
    return path

def mkpar(path, *args, **kwargs):
    if isinstance(path, Path):
        path.parent.mkdir(exist_ok=True, parents=True)
    else:
        mkdir(dirname(path))
    return path


# ### Other

# In[ ]:

from configparser import ConfigParser
from dataclasses import dataclass

try:
    # Python 3.8
    from functools import cached_property, singledispatchmethod
except ImportError:
    try:
        # Python ≤3.7; pip install cached-property
        from cached_property import cached_property
    except ImportError as e:
        pass

from functools import partial, lru_cache, namedtuple, reduce, singledispatch

from glob import glob

from itertools import combinations, combinations_with_replacement, permutations

import json

import os
from os import cpu_count, environ as env, getcwd
from os.path import basename, dirname, exists, join, splitext

from re import match

import shlex
from shutil import copy, copyfileobj, move, rmtree

from subprocess import check_call, check_output, CalledProcessError, DEVNULL, PIPE, Popen

from sys import stdout, stderr, executable as python

from tempfile import NamedTemporaryFile, TemporaryDirectory, TemporaryFile
from time import sleep


# ## Sibling modules
# Some other notebooks and Python files from this repo:

# In[3]:


from . import process
from .process import *

from . import pnds
from .pnds import *

from .cd import cd
from .o import o

from .git import Git, Repo, make_repo

from .args_parser import *

from .context import *

from .collections import coerce, singleton


# ## Optional Modules

# In[8]:


# joblib: easy parallelization
with _try:
    from joblib import Parallel, delayed
    parallel = Parallel(n_jobs=cpu_count())

# yaml
with _try:
    import yaml
    # Fix a bad default in PyYAML (cf. https://github.com/yaml/pyyaml/issues/110)
    yaml.dump = partial(yaml.dump, sort_keys=False)
    yaml.safe_dump = partial(yaml.safe_dump, sort_keys=False)

# requests
with _try:
    from requests import           get as   GET,          post as  POST,           put as   PUT,         patch as PATCH


# ## PyData / Scientific Python

# In[9]:


import numpy as np
from numpy import concatenate, array, ndarray, matrix, nan


# In[10]:


with _try: import seaborn as sns
with _try: import matplotlib.pyplot as plt
with _try: from scipy.sparse import spmatrix, coo_matrix, csr_matrix, csc_matrix




#!/usr/bin/env python
# coding: utf-8

# In[2]:


import shlex
from subprocess import check_call, check_output, CalledProcessError, DEVNULL
from collections.abc import Iterable


def parse_cmd(cmd):
    '''Stringify and potentially unwrap a command'''
    if len(cmd) == 1         and isinstance(cmd[0], Iterable)         and not isinstance(cmd[0], str):
        cmd = cmd[0]
    return [
        str(arg)
        for arg in cmd
        if arg is not None
    ]


def lines(*cmd, keep_trailing_newline=False, dry_run=False, **kwargs):
    '''Return the lines written to stdout by a command'''
    lines = [
        line.rstrip('\n')
        for line in
        output(*cmd, dry_run=dry_run, **kwargs).decode().split('\n')
    ]

    if not keep_trailing_newline and lines and not lines[-1]:
        lines = lines[:-1]

    return lines


def line(*cmd, **kwargs):
    '''Run a command, verify that it returns a single line of output, and return that line'''
    _lines = lines(*cmd, **kwargs)
    if len(_lines) == 1:
        return _lines[0]
    else:
        raise ValueError(f'Expected 1 line, found {len(_lines)}:\n\t%s' % '\n\t'.join(_lines))

def run(*cmd, dry_run=False, **kwargs):
    '''Convenience wrapper for check_call'''
    cmd = parse_cmd(cmd)
    shlex_join = getattr(shlex, 'join', ' '.join)  # shlex.join only exists in Python ≥3.8
    if dry_run:
        print(f'Would run: {shlex_join(cmd)}')
    else:
        print(f'Running: {shlex_join(cmd)}')
        check_call(cmd, **kwargs)

sh = run

def output(*cmd, dry_run=False, **kwargs):
    '''Convenience wrapper for check_output'''
    cmd = parse_cmd(cmd)
    shlex_join = getattr(shlex, 'join', ' '.join)  # shlex.join only exists in Python ≥3.8
    if dry_run:
        print(f'Would run: {shlex_join(cmd)}')
        return b''
    else:
        print(f'Running: {shlex_join(cmd)}')
        return check_output(cmd, **kwargs)


from json import loads
def json(*cmd, **kwargs):
    '''Run a command, parse the output as JSON, and return the parsed object'''
    return loads(output(*cmd, **kwargs).decode())


def check(*cmd, stdout=DEVNULL, stderr=DEVNULL):
    '''Return True iff a command run successfully (i.e. exits with code 0)'''
    try:
        run(*cmd, stdout=stdout, stderr=stderr)
        return True
    except CalledProcessError:
        return False




from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="utz",
    version="0.0.6",
    author="Ryan Williams",
    author_email="ryan@runsascoded.com",
    description="Misc stdlib, pandas, subprocess, and other utilities, exposed for easy importing + boilerplate-reduction",
    install_requires=[
        "GitPython",
        "joblib",
        "pandas",
        "pyyaml",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/runsascoded/utz",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
